
# q-expansion of newform 1144.2.a.g, downloaded from the LMFDB on 06 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1144
weight = 2
poly_data = [-1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [-1, -1], [-2, 2], [2, -3], [1, 0], [1, 0], [-2, 4], [3, -3], [-4, 3], [-2, 0], [-6, 2], [4, -4], [-10, 1], [-6, 4], [2, 2], [-1, 5], [-2, -2], [-6, 0], [-6, -4], [-10, -2], [-11, 1], [-12, 4], [10, 3], [4, -14], [-14, 8], [2, -6], [4, -11], [-16, 4], [7, -1], [7, 1], [4, 4], [10, -2], [-16, 6], [0, 4], [6, -1], [4, -8], [12, 3], [0, 2], [15, -7], [10, -14], [0, 4], [-13, 9], [11, -17], [-2, -5], [-9, 15], [1, 5], [-12, 8], [-10, 4], [3, 13], [22, -4], [-12, 2], [-11, 11], [2, 11], [-8, 3], [16, -3], [12, -16], [12, -3], [-5, 13], [12, -22], [-11, -11], [-14, -8], [-14, 0], [-8, 16], [9, -15], [20, -11], [-2, -18], [10, -12], [12, -6], [4, -12], [-18, 13], [10, -18], [26, -13], [-23, -3], [26, 2], [-4, -14], [14, -6], [-7, 23], [-4, 8], [-34, -2], [10, 4], [-12, -5], [2, 16], [-14, -3], [7, 1], [2, 8], [-16, -5], [-24, 6], [-3, -15], [25, -11], [-22, 28], [-16, 28], [16, 0], [-2, 0], [-8, 2], [-18, 26], [2, 16], [20, 0], [-20, 11], [-16, 32], [-2, -7], [2, -8], [-38, -1], [12, -34], [8, 18], [0, 20], [-4, -20], [-16, 18], [-6, -1], [11, 15], [-14, 6], [32, -6], [17, 13], [2, -20], [8, -22], [-16, 14], [-8, 3], [24, -14], [-3, 21], [22, -24], [-2, -10], [-12, -8], [46, -6], [-28, 8], [26, -10], [-44, 0], [-40, 0], [18, -26], [0, 16], [28, -3], [14, 9], [-6, -13], [-24, 44], [-13, 3], [40, -7], [-2, 15], [-14, 31], [-6, -16], [-6, -21], [14, -4], [-12, 38], [42, -1], [2, 12], [-20, 23], [-23, -9], [4, 31], [-22, 0], [37, -31], [18, -16], [15, -5], [-4, -2], [27, 15], [-1, 33], [23, -45], [0, -6], [16, -33], [-32, 3], [-34, -8], [34, -38], [26, -30], [13, -35], [-6, 0], [-4, 22], [7, 5], [25, 9], [-16, 8], [12, -26], [-21, -17], [10, -24], [-30, 45], [38, -12], [-14, 13], [8, 0], [20, 12], [22, -40], [24, -21], [44, 9], [-2, 25], [0, -20], [10, 32], [-6, 40], [-22, 26], [32, 2], [41, -17], [46, -24], [16, -40], [16, 31], [6, 26], [-44, 4], [51, -9], [-45, 29], [36, -9], [-6, 21], [-7, -35], [-8, -8], [-14, 4], [-47, 15], [-31, 45], [-13, 41], [-10, 16], [-10, 38], [10, -24], [-20, -4], [28, 12], [46, -30], [32, -32], [-27, 35], [-24, 22], [-48, 34], [17, 1], [-52, 20], [-8, 20], [-14, -24], [-16, -16], [-14, 6], [-30, -25], [-4, 0], [42, -6], [-4, 18], [-16, 54], [8, -10], [26, -38], [24, -36], [8, -12], [-50, -8], [2, 24], [-46, 36], [48, -15], [-40, -16], [28, -52], [-9, -19], [17, 23], [0, 0], [-15, 7], [-12, -6], [8, -40], [-5, -31], [-36, 42], [19, -9], [1, -7], [24, -15], [12, 26], [8, 22], [28, 28], [-32, 32], [15, -25], [50, -18], [-20, -16], [-44, 16], [11, -51], [32, -8], [-50, 12], [-39, 17], [-30, 58], [24, -33], [-6, 30], [-57, 21], [4, 7], [-46, 33], [52, 1], [-42, 23], [-22, -4], [16, 12], [6, 1], [-32, 40], [40, -46], [-15, 41], [-2, 12], [-26, 2], [-44, 24], [28, 0], [-40, -5], [10, 4], [-56, 20], [2, -8], [-16, -30], [-28, -2], [32, -42], [58, -39], [2, 23], [18, -30], [-46, 34], [-14, 24], [4, -2], [-26, -17], [14, 11], [27, -9], [1, 11], [-2, 13], [50, -4], [-4, 18], [16, 9], [44, -16], [7, 11], [-15, 11], [30, 20], [-24, -14], [-68, 5], [11, 17], [38, -8], [41, -25], [16, 20], [-30, 33], [-36, 20], [-53, -21], [-20, -25], [-84, 6], [-16, 13], [32, -39], [-52, 28], [-64, 10], [-6, -36], [-32, 4], [-40, 0], [52, 5], [62, -29], [-12, -22], [-70, 14], [-10, 45], [4, -32], [20, 8], [-6, -38], [42, 30], [-30, -26], [48, 18], [-42, 36], [-68, 15], [-38, 4], [0, -22], [-30, -4], [-74, 6], [46, -36], [-69, 5], [-12, 32], [2, -9], [-18, -9], [46, -6], [-48, 28], [18, -4], [-56, 56], [-62, 8], [-38, 60], [18, -44], [-29, 47], [-68, 16], [-22, -30], [10, -8], [9, -13], [-24, -36], [-11, -49], [35, 25], [-6, 57], [6, -38], [72, -8], [-34, 12], [-6, 18], [14, 26], [-40, 48], [44, -28], [18, 8], [30, 18], [-18, 15], [-22, -36], [20, -48], [-10, -45], [-28, 13], [3, 45], [-26, 13], [35, -27], [42, -34], [40, -29], [33, -35], [-18, -39], [60, -64], [38, 34], [13, 11], [-34, 78], [8, 24], [6, -20], [-42, 36], [-34, -38], [-12, 54], [80, 8], [20, 16], [-34, -35], [39, 5], [20, 8], [-60, 66], [-45, -7], [72, 8], [8, 13], [-36, -28], [28, -12], [-40, 0], [-40, 53], [12, -58], [-54, -28], [51, -31], [-6, 52], [24, -4], [-70, 27], [14, -33], [-24, 17], [-13, 17], [44, 2], [-31, -39], [-32, -8], [28, -12], [2, 27], [8, -58], [30, -12], [-58, -26], [-2, 44], [33, -35], [40, 6], [52, -46], [-32, -27], [-63, 43], [-48, 92], [-18, 32], [-42, 48], [8, -27], [-18, -11], [40, -14], [-54, -6], [20, -22], [-68, 40], [-60, 29], [6, -24], [74, -18], [-40, 30]]
